@@ -6,6 +6,8 @@ import signal
 import threading
 import traceback
 
+import psutil
+
 from adam.config import Config
 from adam.sso.idp import Idp
 from adam.app_session import AppSession, IdpLogin
@@ -23,11 +25,62 @@ class TokenHandler(BaseHTTPRequestHandler):
         pass
 
     def do_GET(self):
+        print('cient', self.client_address)
+        print('ports', TokenHandler.get_ports())
         self.send_response(200)
         self.send_header('Content-type', 'text/plain')
         self.end_headers()
         self.wfile.write(self.idp_token.encode('utf8'))
 
+    def get_ports():
+        ports = []
+
+        for p in TokenHandler.get_all_child_processes():
+            ports.extend(TokenHandler.get_open_ports_by_pid(p.pid))
+
+        return ports
+
+    def get_all_child_processes(parent_pid:int=None) -> list:
+        """
+        Retrieves all child processes of a given parent PID.
+        If parent_pid is None, it gets children of the current process.
+        """
+        if parent_pid is None:
+            parent_pid = os.getpid() # Get PID of the current Python script
+
+        try:
+            parent_process = psutil.Process(parent_pid)
+            # children(recursive=True) gets all descendants, not just direct children
+            children = parent_process.children(recursive=True)
+            return children
+        except psutil.NoSuchProcess:
+            print(f"Process with PID {parent_pid} not found.")
+            return []
+        except psutil.AccessDenied:
+            print(f"Access denied to process with PID {parent_pid}.")
+            return []
+
+    def get_open_ports_by_pid(pid):
+        """
+        Retrieves a list of open network ports associated with a given process ID.
+        """
+        try:
+            process = psutil.Process(pid)
+            connections = process.connections()
+            open_ports = []
+            for conn in connections:
+                # Filter for listening or established connections on IPv4 or IPv6
+                if conn.status in [psutil.CONN_LISTEN, psutil.CONN_ESTABLISHED]:
+                    if conn.laddr and conn.laddr.port:
+                        open_ports.append(conn.laddr.port)
+            return list(set(open_ports)) # Return unique ports
+        except psutil.NoSuchProcess:
+            print(f"Error: No process found with PID {pid}")
+            return []
+        except psutil.AccessDenied:
+            print(f"Error: Access denied to process with PID {pid}. Run as administrator/root.")
+            return []
+
 class UserEntry(Command):
     COMMAND = 'entry'
 
