@@ -90,7 +90,8 @@ def enter_repl(state: ReplState):
             completer = NestedCompleter.from_nested_dict({})
             if not state.bash_session:
                 completions = {}
-                if state.app_app:
+                # app commands are available only on a: drive
+                if state.device == ReplState.A and state.app_app:
                     completions = Apps(path='apps.yaml').commands()
 
                 for cmd in sorted_cmds:
@@ -99,8 +100,7 @@ def enter_repl(state: ReplState):
                         completions = deep_merge_dicts(completions, cmd.completion(state))
                     finally:
                         if Config().get('debug.timings', False):
-                            print('Timing completion calc', cmd.command(), f'{time.time() - s1:.2f}')
-                        pass
+                            Config().debug('Timing completion calc', cmd.command(), f'{time.time() - s1:.2f}')
 
                 completer = NestedCompleter.from_nested_dict(completions)
 
